# Copyright 2010-2014 Michael Frank <msfrank@syntaxjockey.com>
#
# This file is part of Pesky.  Pesky is BSD-licensed software;
# for copyright information see the LICENSE file.

import os, json
from setuptools import Command
from distutils import log
from pkg_resources import safe_name, to_filename


class pesky_default(Command):

    # Brief (40-50 characters) description of the command
    description = "manipulate pesky defaults"

    # List of option tuples: long name, short name (None if no short
    # name), and help string.
    user_options = [
        ('command=', None, "perform the specified COMMAND"),
        ('key=', None, "manipulate default named KEY"),
        ('value=', None, "set default to the specified VALUE"),
        ]

    def initialize_options(self):
        self.egg_name = None
        self.egg_base = None
        self.defaults_path = None
        self.command = None
        self.key = None
        self.value = None

    def finalize_options(self):
        self.egg_name = safe_name(self.distribution.get_name())
        log.debug("egg_name = " + self.egg_name)
        if self.egg_base is None:
            dirs = self.distribution.package_dir
            self.egg_base = (dirs or {}).get('',os.curdir)
        log.debug("egg_base = " + self.egg_base)
        self.ensure_dirname('egg_base')
        self.egg_info = to_filename(self.egg_name) + '.egg-info'
        if self.egg_base != os.curdir:
            self.egg_info = os.path.join(self.egg_base, self.egg_info)
        log.debug("egg_info = " + self.egg_info)
        self.defaults_path = os.path.join(self.egg_info, "pesky_defaults.json")

    def read_defaults(self):
        if not os.access(self.defaults_path, os.F_OK):
            return dict()
        with open(self.defaults_path, 'r') as f:
            return json.loads(f.read())

    def write_defaults(self, defaults):
        log.debug("writing pesky defaults to %s", self.defaults_path)
        if not self.dry_run:
            with open(self.defaults_path, 'w') as f:
                f.write(json.dumps(defaults))

    def run(self):
        if self.command is None:
            log.info("no command specified")
            return 0
        command = self.command.lower()
        defaults = self.read_defaults()
        # commands which don't require a key
        if command == 'list':
            for key,value in sorted(defaults.items()):
                print "%s: %s" % (key, str(value))
            return 0
        if command == 'flush':
            log.debug("flushing all defaults")
            self.write_defaults(dict())
            return 0
        # commands which require a key
        if self.key is None:
            log.info("no key specified")
            return 0
        if command == 'get':
            if self.key in defaults:
                print defaults[self.key]
            return 0
        if command == 'set':
            log.debug("setting %s = '%s'" % (self.key, self.value))
            defaults[self.key] = self.value
            self.write_defaults(defaults)
            return 0
        if command == 'clear':
            log.debug("clearing %s" % self.key)
            if self.key in defaults:
                del defaults[self.key]
                self.write_defaults(defaults)
            return 0
        log.info("unknown command %s" % self.command)
        return 1



# Copyright 2010-2014 Michael Frank <msfrank@syntaxjockey.com>
#
# This file is part of Pesky.  Pesky is BSD-licensed software;
# for copyright information see the LICENSE file.

import platform

def system_defaults():
    system = platform.system().lower()
    if system == 'linux':
        return {
            'PREFIX': '/usr',
            'BIN_DIR': '/usr/bin',
            'SBIN_DIR': '/usr/sbin',
            'LIB_DIR': '/usr/lib',
            'LIBEXEC_DIR': '/usr/libexec',
            'SYSCONF_DIR': '/etc',
            'LOCALSTATE_DIR': '/var/lib',
            'RUN_DIR': '/var/run',
            'DATA_DIR': '/usr/share',
        }
    elif system == 'darwin':
        return {
            'PREFIX': '/usr',
            'BIN_DIR': '/usr/bin',
            'SBIN_DIR': '/usr/sbin',
            'LIB_DIR': '/usr/lib',
            'LIBEXEC_DIR': '/usr/libexec',
            'SYSCONF_DIR': '/etc',
            'LOCALSTATE_DIR': '/var/lib',
            'RUN_DIR': '/var/run',
            'DATA_DIR': '/usr/share',
        }
    elif system == 'windows':
        return dict()
    else:
        return dict()



# Copyright 2010-2014 Michael Frank <msfrank@syntaxjockey.com>
#
# This file is part of Pesky.  Pesky is BSD-licensed software;
# for copyright information see the LICENSE file.

class DefaultsError(Exception):
    pass

class UndefinedDefault(DefaultsError):
    pass



# Copyright 2010-2014 Michael Frank <msfrank@syntaxjockey.com>
#
# This file is part of Pesky.  Pesky is BSD-licensed software;
# for copyright information see the LICENSE file.

import pkg_resources, json

def package_defaults(project):
    req = pkg_resources.Requirement.parse(project)
    provider = pkg_resources.get_provider(req)
    if provider.has_metadata('pesky_defaults.json'):
        return json.loads(provider.get_metadata('pesky_defaults.json'))
    return dict()



# Copyright 2010-2014 Michael Frank <msfrank@syntaxjockey.com>
#
# This file is part of Pesky.  Pesky is BSD-licensed software;
# for copyright information see the LICENSE file.

import sys, collections

from pesky.defaults.errors import UndefinedDefault, DefaultsError

class Defaults(collections.Mapping):
    """
    Contains defaults data.  The defaults are obtained in the following order:
    First, entries are loaded from system/platform defaults (linux, windows, mac).
    Second, if a site module has been defined (such as by a distribution-specific
    package), then these entries override the system defaults, unless ignoresite
    is set to True.  Finally, if project is not None, then entries are loaded
    from the specified project pkg_resources.
    """
    def __init__(self, project=None, ignoresite=False):
        self._defaults = dict()
        # load system defaults
        from pesky.defaults.system import system_defaults
        for name,value in system_defaults().items():
            self._defaults[name] = value
        # load site overrides if the module exists
        if not ignoresite:
            try:
                from pesky.defaults.site import site_defaults
                for name,value in site_defaults().items():
                    self._defaults[name] = value
            except ImportError, e:
                pass
        # load package overrides
        if project is not None:
            from pesky.defaults.package import package_defaults
            for name,value in package_defaults(project).items():
                self._defaults[name] = value

    def get(self, name):
        """
        Get the default with the specified name.  if the entry doesn't exist,
        then raise UndefinedDefault exception.
        """
        try:
            return self._defaults[name]
        except KeyError:
            raise UndefinedDefault("default %s is undefined" % name)

    def getorelse(self, name, default=None):
        """
        Get the default with the specified name.  if the entry doesn't exist,
        then return the default, which is None by default.
        """
        try:
            return self._defaults[name]
        except KeyError:
            return default

    def has(self, name):
        """
        Return True if an entry exists with the specified name, otherwise False.
        """
        return name in self._defaults

    def __getitem__(self, name):
        return self._defaults[name]

    def __iter__(self):
        return iter(self._defaults)

    def __len__(self):
        return len(self._defaults)



#!/usr/bin/env python

from setuptools import setup

# jump through some hoops to get access to versionstring()
from sys import path
from os.path import abspath, dirname
path.insert(0, abspath(dirname(__file__)))
from pesky.defaults import versionstring

setup(
    # package description
    name = "pesky-defaults",
    version = versionstring(),
    description="Pesky configuration defaults interface",
    author="Michael Frank",
    author_email="syntaxockey@gmail.com",
    # installation dependencies
    install_requires=[
        ],
    # package classifiers for PyPI
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License", 
        "Natural Language :: English",
        "Programming Language :: Python :: 2",
        ],
    # package contents
    namespace_packages=[
        "pesky",
        ],
    packages=[
        "pesky",
        'pesky.defaults',
        ],
    # distuils commands
    entry_points = {
        "distutils.commands": [
            "pesky_default = pesky.defaults.distcmd:pesky_default",
            ],
        },
    # test configuration
    test_suite="test",
    tests_require=["nose >= 1.3.1"]
)



# Copyright 2010-2014 Michael Frank <msfrank@syntaxjockey.com>
#
# This file is part of Pesky.  Pesky is BSD-licensed software;
# for copyright information see the LICENSE file.

__version__ = (0, 0, 1)

def versionstring():
    """
    Return the version number as a string.
    """
    return "%i.%i.%i" % __version__


from pesky.defaults.defaults import Defaults
from pesky.defaults.errors import DefaultsError, UndefinedDefault
